@@ -1,8 +1,9 @@
-use std::{env, fmt, io, sync::mpsc, time::Duration};
+use std::{env, fmt, io, path::Path, sync::mpsc, time::Duration};
 
 use crate::{
     buffer::parse_path_and_position,
     client::{ClientHandle, ClientManager},
+    command::{CommandManager, CommandOperation},
     editor::{Editor, EditorControlFlow},
     events::{ClientEvent, ClientEventReceiver, ServerEvent},
     platform::{Key, Platform, PlatformRequest, ProcessHandle, ProcessTag, SharedBuf},
@@ -65,15 +66,17 @@ impl ServerApplication {
         Duration::from_secs(1)
     }
 
-    pub fn run(mut platform: Platform) -> Option<mpsc::Sender<ApplicationEvent>> {
+    pub fn run(args: Args, mut platform: Platform) -> mpsc::Sender<ApplicationEvent> {
         let current_dir = env::current_dir().expect("could not retrieve the current directory");
         let editor = Editor::new(current_dir);
         let clients = ClientManager::new();
 
+        let source_default_config = !args.no_default_config;
         let (event_sender, event_receiver) = mpsc::channel();
         let event_sender_clone = event_sender.clone();
         std::thread::spawn(move || {
             let _ = Self::run_application(
+                source_default_config,
                 editor,
                 clients,
                 &mut platform,
@@ -84,18 +87,35 @@ impl ServerApplication {
             platform.flush_requests();
         });
 
-        Some(event_sender)
+        event_sender
     }
 
     fn run_application(
+        source_default_config: bool,
         mut editor: Editor,
         mut clients: ClientManager,
         platform: &mut Platform,
         event_sender: mpsc::Sender<ApplicationEvent>,
         event_receiver: mpsc::Receiver<ApplicationEvent>,
     ) -> Result<(), AnyError> {
+        let mut is_first_client = true;
         let mut client_event_receiver = ClientEventReceiver::default();
 
+        if source_default_config {
+            let source = include_str!("default_config.pp");
+            match CommandManager::eval_commands_then_output(
+                &mut editor,
+                platform,
+                &mut clients,
+                None,
+                source,
+                Some(Path::new("default_config.pp")),
+            ) {
+                Some(CommandOperation::Quit) | Some(CommandOperation::QuitAll) => return Ok(()),
+                _ => (),
+            }
+        }
+
         'event_loop: loop {
             let mut event = event_receiver.recv()?;
             loop {
@@ -106,10 +126,12 @@ impl ServerApplication {
                         clients.on_client_joined(handle);
                         let mut buf = platform.buf_pool.acquire();
                         let write = buf.write();
+                        write.push(is_first_client as _);
                         write.push(handle.into_index() as _);
                         let buf = buf.share();
                         platform.buf_pool.release(buf.clone());
                         platform.enqueue_request(PlatformRequest::WriteToClient { handle, buf });
+                        is_first_client = false;
                     }
                     ApplicationEvent::ConnectionClose { handle } => {
                         clients.on_client_left(handle);
@@ -206,7 +228,11 @@ impl<'stdout> ClientApplication<'stdout> {
         2 * 1024
     }
 
-    pub fn new(handle: ClientHandle, stdout: io::StdoutLock<'stdout>, is_pipped: bool) -> Self {
+    pub fn new(
+        handle: ClientHandle,
+        stdout: io::StdoutLock<'stdout>,
+        is_pipped: bool,
+    ) -> Self {
         Self {
             handle,
             is_pipped,
@@ -217,7 +243,7 @@ impl<'stdout> ClientApplication<'stdout> {
         }
     }
 
-    pub fn init<'a>(&'a mut self, args: Args) -> &'a [u8] {
+    pub fn init<'a>(&'a mut self, args: Args, is_first_client: bool) -> &'a [u8] {
         self.server_write_buf.clear();
 
         if let Some(handle) = args.as_client {
@@ -225,9 +251,15 @@ impl<'stdout> ClientApplication<'stdout> {
         }
 
         let mut commands = String::new();
-        for config in &args.configs {
-            use fmt::Write;
-            writeln!(commands, "source '{}'", config).unwrap();
+        if is_first_client {
+            for config in &args.configs {
+                use fmt::Write;
+                if config.throw_error {
+                    writeln!(commands, "source '{}'", &config.path).unwrap();
+                } else {
+                    writeln!(commands, "try {{ source '{}' }}", &config.path).unwrap();
+                }
+            }
         }
         for path in &args.files {
             use fmt::Write;
@@ -337,3 +369,4 @@ impl<'stdout> Drop for ClientApplication<'stdout> {
         }
     }
 }
+
