@@ -26,14 +26,20 @@ pub mod theme;
 pub mod ui;
 pub mod word_database;
 
+pub struct ArgsConfig {
+    path: String,
+    throw_error: bool,
+}
+
 #[derive(Default)]
 pub struct Args {
     pub version: bool,
-    pub configs: Vec<String>,
     pub session: Option<String>,
-    pub as_client: Option<client::ClientHandle>,
     pub print_session: bool,
-    pub as_server: bool,
+    pub as_client: Option<client::ClientHandle>,
+    pub server: bool,
+    pub configs: Vec<ArgsConfig>,
+    pub no_default_config: bool,
     pub files: Vec<String>,
 }
 
@@ -49,18 +55,20 @@ fn print_help() {
     println!();
     println!("usage: pepper [<options...>] [<files...>]");
     println!();
-    println!("  files: files to open as a buffer");
+    println!("  files: file paths to open as a buffer");
     println!("         you can append ':<line>[,<column>]' to open it at that position");
     println!();
     println!("options:");
     println!();
-    println!("  -h, --help                prints help and quits");
-    println!("  -v, --version             prints version and quits");
-    println!("  -c, --config              loads config file at path (repeatable)");
-    println!("  -s, --session             session name to connect to");
-    println!("  --as-client <client-id>   sends events as if it was client with id <client-id>");
-    println!("  --print-session           print the computed session name and quits");
-    println!("  --as-server               ignore all other options and only run as server");
+    println!("  -h, --help               prints help and quits");
+    println!("  -v, --version            prints version and quits");
+    println!("  -s, --session            overrides the session name to connect to");
+    println!("  --print-session          prints the computed session name and quits");
+    println!("  --as-client <client-id>  sends events as if it was client with id <client-id>");
+    println!("  --server                 only run as server (ignores files and configs arguments)");
+    println!("  -c, --config             sources config file at path (repeatable)");
+    println!("  --try-config             tries to source a config file at path, throws no error if fails (repeatable)");
+    println!("  --no-default-config      does not source the default config included in the editor");
 }
 
 impl Args {
@@ -92,13 +100,6 @@ impl Args {
                     print_version();
                     std::process::exit(0);
                 }
-                "-c" | "--config" => match args.next() {
-                    Some(arg) => {
-                        let arg = arg_to_str(&arg);
-                        parsed.configs.push(arg.into());
-                    }
-                    None => error(format_args!("expected config path after {}", arg)),
-                },
                 "-s" | "--session" => match args.next() {
                     Some(arg) => {
                         let arg = arg_to_str(&arg);
@@ -111,6 +112,7 @@ impl Args {
                     }
                     None => error(format_args!("expected session after {}", arg)),
                 },
+                "--print-session" => parsed.print_session = true,
                 "--as-client" => match args.next() {
                     Some(arg) => {
                         let arg = arg_to_str(&arg);
@@ -124,8 +126,28 @@ impl Args {
                     }
                     None => error(format_args!("expected client id after {}", arg)),
                 },
-                "--print-session" => parsed.print_session = true,
-                "--as-server" => parsed.as_server = true,
+                "--server" => parsed.server = true,
+                "-c" | "--config" => match args.next() {
+                    Some(arg) => {
+                        let arg = arg_to_str(&arg);
+                        parsed.configs.push(ArgsConfig {
+                            path: arg.into(),
+                            throw_error: true,
+                        });
+                    }
+                    None => error(format_args!("expected config path after {}", arg)),
+                },
+                "--try-config" => match args.next() {
+                    Some(arg) => {
+                        let arg = arg_to_str(&arg);
+                        parsed.configs.push(ArgsConfig {
+                            path: arg.into(),
+                            throw_error: false,
+                        });
+                    }
+                    None => error(format_args!("expected config path after {}", arg)),
+                },
+                "--no-default-config" => parsed.no_default_config = true,
                 "--" => {
                     while let Some(arg) = args.next() {
                         let arg = arg_to_str(&arg);
