@@ -246,11 +246,7 @@ fn update_autocomplete_entries(ctx: &mut ModeContext) {
         CompletionSource::Files => {
             fn set_files_in_path_as_entries(picker: &mut Picker, path: &str) {
                 picker.clear();
-                let path = if path.is_empty() {
-                    "."
-                } else {
-                    path
-                };
+                let path = if path.is_empty() { "." } else { path };
                 let read_dir = match fs::read_dir(path) {
                     Ok(iter) => iter,
                     Err(_) => return,
@@ -286,4 +282,3 @@ fn update_autocomplete_entries(ctx: &mut ModeContext) {
     state.completion_source = completion_source;
     ctx.editor.picker.filter(WordIndicesIter::empty(), pattern);
 }
-
