@@ -114,9 +114,9 @@ pub fn main() {
     let input_handle = get_std_handle(STD_INPUT_HANDLE);
     let output_handle = get_std_handle(STD_OUTPUT_HANDLE);
 
-    if args.as_server {
+    if args.server {
         if !pipe_exists(&pipe_path) {
-            let _ = run_server(&pipe_path);
+            let _ = run_server(args, &pipe_path);
         }
     } else if let Some(input_handle) = input_handle {
         if !pipe_exists(&pipe_path) {
@@ -294,7 +294,7 @@ fn read(handle: &Handle, buf: &mut [u8]) -> Result<usize, usize> {
     let result = unsafe {
         ReadFile(
             handle.0,
-            buf.as_ptr() as _,
+            buf.as_mut_ptr() as _,
             buf.len() as _,
             &mut read_len,
             std::ptr::null_mut(),
@@ -360,7 +360,7 @@ fn fork() {
             command_line.push(short);
         }
     }
-    command_line.extend(b" --as-server".iter().map(|&b| b as u16));
+    command_line.extend(b" --server".iter().map(|&b| b as u16));
     command_line.push(0);
 
     let result = unsafe {
@@ -612,13 +612,13 @@ impl Drop for ConsoleMode {
 
 struct ConnectionToClient {
     reader: AsyncReader,
-    read_buf: Option<ExclusiveBuf>,
+    current_buf: Option<ExclusiveBuf>,
 }
 impl ConnectionToClient {
     pub fn new(reader: AsyncReader) -> Self {
         Self {
             reader,
-            read_buf: None,
+            current_buf: None,
         }
     }
 
@@ -631,22 +631,27 @@ impl ConnectionToClient {
         buf_len: usize,
         buf_pool: &mut BufPool,
     ) -> Result<Option<SharedBuf>, ()> {
-        let mut read_buf = match self.read_buf.take() {
+        let mut buf = match self.current_buf.take() {
             Some(buf) => buf,
             None => buf_pool.acquire(),
         };
-        let buf = read_buf.write_with_len(buf_len);
+        let write = buf.write_with_len(buf_len);
 
-        match self.reader.read_async(buf) {
+        match self.reader.read_async(write) {
             ReadResult::Waiting => {
-                self.read_buf = Some(read_buf);
+                self.current_buf = Some(buf);
                 Ok(None)
             }
             ReadResult::Ok(len) => {
-                buf.truncate(len);
-                Ok(Some(read_buf.share()))
+                write.truncate(len);
+                let buf = buf.share();
+                buf_pool.release(buf.clone());
+                Ok(Some(buf))
+            }
+            ReadResult::Err => {
+                buf_pool.release(buf.share());
+                Err(())
             }
-            ReadResult::Err => Err(()),
         }
     }
 
@@ -732,7 +737,7 @@ impl ConnectionToClientListener {
 struct ProcessPipe {
     reader: AsyncReader,
     buf_len: usize,
-    current_read_buf: Option<ExclusiveBuf>,
+    current_buf: Option<ExclusiveBuf>,
 }
 impl ProcessPipe {
     pub fn new(reader: AsyncReader, buf_len: usize) -> Self {
@@ -741,7 +746,7 @@ impl ProcessPipe {
         Self {
             reader,
             buf_len,
-            current_read_buf: None,
+            current_buf: None,
         }
     }
 
@@ -750,22 +755,27 @@ impl ProcessPipe {
     }
 
     pub fn read_async(&mut self, buf_pool: &mut BufPool) -> Result<Option<SharedBuf>, ()> {
-        let mut read_buf = match self.current_read_buf.take() {
+        let mut buf = match self.current_buf.take() {
             Some(buf) => buf,
             None => buf_pool.acquire(),
         };
-        let buf = read_buf.write_with_len(self.buf_len);
+        let write = buf.write_with_len(self.buf_len);
 
-        match self.reader.read_async(buf) {
+        match self.reader.read_async(write) {
             ReadResult::Waiting => {
-                self.current_read_buf = Some(read_buf);
+                self.current_buf = Some(buf);
                 Ok(None)
             }
             ReadResult::Ok(len) => {
-                buf.truncate(len);
-                Ok(Some(read_buf.share()))
+                write.truncate(len);
+                let buf = buf.share();
+                buf_pool.release(buf.clone());
+                Ok(Some(buf))
+            }
+            ReadResult::Err => {
+                buf_pool.release(buf.share());
+                Err(())
             }
-            ReadResult::Err => Err(()),
         }
     }
 }
@@ -849,7 +859,7 @@ impl Events {
 
     pub fn track(&mut self, event: &Event, source: EventSource) {
         let index = self.len;
-        debug_assert!(index < MAX_EVENT_COUNT);
+        debug_assert!(index < self.wait_handles.len());
         self.wait_handles[index] = event.handle();
         self.sources[index] = source;
         self.len += 1;
@@ -865,7 +875,7 @@ impl Events {
     }
 }
 
-fn run_server(pipe_path: &[u16]) -> Result<(), AnyError> {
+fn run_server(args: Args, pipe_path: &[u16]) -> Result<(), AnyError> {
     const NONE_ASYNC_PROCESS: Option<AsyncProcess> = None;
     static NEW_REQUEST_EVENT_HANDLE: AtomicPtr<()> = AtomicPtr::new(std::ptr::null_mut());
 
@@ -873,10 +883,6 @@ fn run_server(pipe_path: &[u16]) -> Result<(), AnyError> {
     let mut listener =
         ConnectionToClientListener::new(pipe_path, ServerApplication::connection_buffer_len());
 
-    let mut client_connections: [Option<ConnectionToClient>; MAX_CLIENT_COUNT] = Default::default();
-    let mut processes = [NONE_ASYNC_PROCESS; MAX_PROCESS_COUNT];
-    let mut buf_pool = BufPool::default();
-
     let new_request_event = Event::automatic();
     NEW_REQUEST_EVENT_HANDLE.store(new_request_event.0 as _, Ordering::Relaxed);
 
@@ -886,11 +892,11 @@ fn run_server(pipe_path: &[u16]) -> Result<(), AnyError> {
         request_sender,
     );
     platform.set_clipboard_api(read_from_clipboard, write_to_clipboard);
+    let event_sender = ServerApplication::run(args, platform);
 
-    let event_sender = match ServerApplication::run(platform) {
-        Some(sender) => sender,
-        None => return Ok(()),
-    };
+    let mut client_connections: [Option<ConnectionToClient>; MAX_CLIENT_COUNT] = Default::default();
+    let mut processes = [NONE_ASYNC_PROCESS; MAX_PROCESS_COUNT];
+    let mut buf_pool = BufPool::default();
 
     let mut timeout = Some(ServerApplication::idle_duration());
 
@@ -1161,18 +1167,20 @@ impl ConnectionToServer {
 fn run_client(args: Args, pipe_path: &[u16], input_handle: Handle, output_handle: Option<Handle>) {
     let mut connection = ConnectionToServer::connect(pipe_path);
 
-    let mut client_index = 0;
-    match connection.read(std::slice::from_mut(&mut client_index)) {
-        Ok(1) => (),
+    let mut buf = [0; 2];
+    match connection.read(&mut buf) {
+        Ok(2) => (),
         _ => return,
     }
+    let is_first_client = buf[0] != 0;
+    let client_index = buf[1];
 
     let client_handle = ClientHandle::from_index(client_index as _).unwrap();
     let is_pipped = is_pipped(&input_handle);
 
     let stdout = io::stdout();
     let mut application = ClientApplication::new(client_handle, stdout.lock(), is_pipped);
-    let bytes = application.init(args);
+    let bytes = application.init(args, is_first_client);
     if !connection.write(bytes) {
         return;
     }
@@ -1334,3 +1342,4 @@ fn parse_console_events(
         }
     }
 }
+
